import 'package:feature_manager/feature_manager.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:shared_preferences/shared_preferences.dart';

void main() {
  late FeatureManager _target;

  setUp(() async {
    SharedPreferences.setMockInitialValues(<String, Object>{});
    _target = FeatureManager();
  });

  group('isEnabled', () {
    test('when feature is toggle should return value', () async {
      // given
      SharedPreferences.setMockInitialValues(<String, Object>{
        'key': true,
      });
      final Feature feature = Feature(
        key: 'key',
        title: '',
        type: FeatureType.feature,
        valueType: FeatureValueType.toggle,
      );

      // when
      final bool value = await _target.isEnabled(feature);

      // then
      expect(value, true);
    });

    test('when feature is toggle and value is null should return default value',
        () async {
      // given
      final Feature feature = Feature(
        key: 'key',
        title: '',
        type: FeatureType.feature,
        valueType: FeatureValueType.toggle,
        defaultValue: true,
      );

      // when
      final bool value = await _target.isEnabled(feature);

      // then
      expect(value, true);
    });

    test(
        'when feature is toggle and value is null and default value is null should return false',
        () async {
      // given
      final Feature feature = Feature(
        key: 'key',
        title: '',
        type: FeatureType.feature,
        valueType: FeatureValueType.toggle,
      );

      // when
      final bool value = await _target.isEnabled(feature);

      // then
      expect(value, false);
    });

    test('when feature is not toggle should return false', () async {
      // given
      final Feature feature = Feature(
        key: 'key',
        title: '',
        type: FeatureType.feature,
        valueType: FeatureValueType.text,
      );

      // when
      final bool value = await _target.isEnabled(feature);

      // then
      expect(value, false);
    });
  });

  group('getValue', () {
    test('return stored value', () async {
      // given
      SharedPreferences.setMockInitialValues(<String, Object>{
        'key': 'some text',
      });
      final Feature feature = Feature(
        key: 'key',
        title: '',
        type: FeatureType.feature,
        valueType: FeatureValueType.text,
      );

      // when
      final String value = await _target.getValue(feature) as String;

      // then
      expect(value, 'some text');
    });

    test('when stored value is null should return default value', () async {
      // given
      final Feature feature = Feature(
        key: 'key',
        title: '',
        type: FeatureType.feature,
        valueType: FeatureValueType.text,
        defaultValue: 'default text',
      );

      // when
      final String value = await _target.getValue(feature) as String;

      // then
      expect(value, 'default text');
    });
  });
}
